import { AnyStrictValidator, InferType, LooseTest, LooseValidator, StrictTest, StrictValidator, Trait } from './types';
export declare function makeTrait<U>(value: U): <V>() => U & Trait<V>;
export declare function makeValidator<U, V extends U>({ test }: {
    test: StrictTest<U, V>;
}): StrictValidator<U, V>;
export declare function makeValidator<U, V extends U = U>({ test }: {
    test: LooseTest<U>;
}): LooseValidator<U, V>;
export declare class TypeAssertionError extends Error {
    constructor({ errors }?: {
        errors?: string[];
    });
}
/**
 * Check that the specified value matches the given validator, and throws an
 * exception if it doesn't. Refine the type if it passes.
 */
export declare function assert<T extends AnyStrictValidator>(val: unknown, validator: T): asserts val is InferType<T>;
/**
 * Check that the specified value matches the given validator, and throws an
 * exception if it doesn't. Refine the type if it passes.
 *
 * Thrown exceptions include details about what exactly looks invalid in the
 * tested value.
 */
export declare function assertWithErrors<T extends AnyStrictValidator>(val: unknown, validator: T): asserts val is InferType<T>;
/**
 * Compile-time only. Refine the type as if the validator was matching the
 * tested value, but doesn't actually run it. Similar to the classic `as`
 * operator in TypeScript.
 */
export declare function softAssert<T extends AnyStrictValidator>(val: unknown, validator: T): asserts val is InferType<T>;
/**
 * Check that the value matches the given validator, and returns undefined
 * if it doesn't instead of throwing an error.
 */
export declare function as<T extends AnyStrictValidator>(val: unknown, validator: T, { coerce }?: {
    coerce?: boolean;
}): InferType<T> | undefined;
declare type FnValidatedArgument<T extends [] | [AnyStrictValidator, ...AnyStrictValidator[]]> = T extends [AnyStrictValidator, ...AnyStrictValidator[]] ? {
    [K in keyof T]: InferType<T[K]>;
} : [];
interface FnValidatedFunction<T extends [] | [AnyStrictValidator, ...AnyStrictValidator[]], Ret> {
    (...args: FnValidatedArgument<T>): Ret;
}
/**
 * Create and return a new function that apply the given validators to each
 * corresponding argument passed to the function and throws an exception in
 * case of a mismatch.
 */
export declare function fn<T extends [] | [AnyStrictValidator, ...AnyStrictValidator[]], Ret>(validators: T, fn: (...args: FnValidatedArgument<T>) => Ret): FnValidatedFunction<T, Ret>;
export {};
